#ifndef TYPES_H
#define TYPES_H

// Derived types.

#include "lotypes.h"
#include <limits.h>
#include <float.h>
#include <list>
#include <algorithm>
#include <vector>
#include <string>

#define	INTEGER_DP		1

// Genome coordinate
typedef uint32 coord_t;

// Number of times a word appears in a genome
typedef unsigned COUNT_TYPE;
const COUNT_TYPE COUNT_TYPE_MAX = UINT_MAX;

const uint32 COORD_T_MAX = UINT_MAX;

// Scores stored in BndFile's and SegPair objects.
typedef int32 sp_score_t;

#if	INTEGER_DP

typedef int score_t;
const score_t SCORE_UNINIT = 911911911;
const score_t MINUS_INF = -5000000;

#define EQ(a, b)	((a) == (b))

#else

typedef float score_t;
const score_t SCORE_UNINIT = (score_t) 911911911;
const score_t MINUS_INF = (score_t) (-1e20);

#define EQ(a, b)	(fabs((a) - (b)) < 0.01)

#endif

// Segment of a genome.
struct Seg
	{
	coord_t lo;
	coord_t hi;

	coord_t Midpoint() const
		{
		return (hi + lo)/2;
		}

	coord_t Length() const
		{
		return hi - lo + 1;
		}

	coord_t Overlap(const Seg &s) const
		{
		coord_t maxlo = std::max(lo, s.lo);
		coord_t minhi = std::min(hi, s.hi);

		if (maxlo > minhi)
			return 0;

		return minhi - maxlo + 1;
		}
	};
class SeqFile;
class ChrNameIndex;

struct CoordData
	{
	unsigned ChrIndex;
	coord_t StartPos;
	coord_t EndPos;
	bool Reversed;

	bool operator==(const CoordData &rhs) const
		{
		return ChrIndex == rhs.ChrIndex && StartPos == rhs.StartPos &&
		  EndPos == rhs.EndPos && Reversed == rhs.Reversed;
		}
	};

struct AlnRow
	{
	std::string Label;
	std::string Letters;
	};

typedef std::vector<CoordData> CoordsVec;
typedef std::vector<const char *> StrVec;
typedef std::vector<std::string> StdStrVec;
typedef std::vector<ChrNameIndex *> ChrNameIndexVec;
typedef std::vector<SeqFile *> SeqFileVec;
typedef std::vector<AlnRow> AlnRowVec;
typedef std::vector<uint64> Uint64Vec;
typedef std::vector<CoordData> CoordVec;
typedef std::vector<char *> STRVEC;
typedef std::vector<char *> STRLIST;

#endif	// TYPES_H
